/*
** RAW_loader.cpp  -  Nikolaj Thygesen  -  21.  1998.
** Loader til fiktivt RAW-format. Bare lis'sem et eksempel
** =======================================================
*/

#include <dos/dos.h>

#include <proto/dos.h>
#include <proto/exec.h>
#include <stdlib.h>
#include <string.h>

struct BitmapLoaderBase {
  struct Library library;
};
#pragma libbase BitmapLoaderBase;

#include "basictype.h"
#include "proto/BitmapLoader.h"
#include "proto/DiamondBox.h"

/*
** Dette er LibBase til DiamondBOX's eget lille lib, der med tiden bliver lidt
** mere ordnet :o) Der er lidt kaos over det PT.
*/
struct Library *DiamondBase;


/*
** Denne struktur hjemmestrikkes til at indeholde de ting og sager man mtte
** behve under lsningen af den enkelte filtype.
*/
struct PebbleHandle {
  unsigned linenumber, width;
  ULONG stream;
};


/*
** Her lser vi en enkelt scanline og returnerer nummeret p den lste scanline,
** i intervallet: [0, height[, i "*lineno". En pointer til vores PebbleHandle
** ligger i "data", og bitmap data smides i "SLP".
*/
enum RCode ASM LoaderScanline(
  A0 struct PebbleHandle *data,
  A1 PixelARGB *SLP,
  A2 unsigned *lineno
)
{
  *lineno = data->linenumber++;
  FRead(data->stream, SLP, PIXELARGB_SZ * data->width, 1);
  return Success;
}


/*
** Allokerer resourcer til lsning af bitmap.
** ==========================================
*/
struct PebbleHandle *OpenLoader(
  A0 char *filename,
  A1 unsigned *picwidth,
  A2 unsigned *picheight,
  D0 struct Library *diamondlib
)
{
  struct PebbleHandle *RC = NULL;
  char ID[4];

  /*
  ** For nemheds skyld overleveres en pointer til lokalt lib, som ogs kunne
  ** bnes. Det giver adgang til en rkke utility-funktioner. Nogle fr er
  ** allerede p nuvrende tidspunkt ligegyldige.
  */
  DiamondBase = diamondlib;
  /*
  ** PebbleHandle SKAL allokeres med dertil indrettet funtion, og ligeledes
  ** frigives med FreePebbleHandle(). Returvrdi forskellig fra NULL angiver
  ** succesfuld bning af bitmap-fil.
  */
  if(RC = AllocPebbleHandle(sizeof(struct PebbleHandle))) {
    if(RC->stream = (ULONG)Open(filename, MODE_OLDFILE)) {
      FRead(RC->stream, ID, 4, 1);
      /*
      ** Bemrk at der ikke fejlmeddeles ifm. identifikation da vi elegant videre-
      ** giver kontrollen til nste loader hvis det viser sig ikke at vre vores
      ** filtype. Vi forestiller os at de frste fire bytes indeholder tegnene:
      ** 'X', 'R', 'A' og 'W' efterfulgt af bredde, hjde og selve bitmap.
      */
      if(!memcmp("XRAW", ID, 4)) {
        FRead(RC->stream, picwidth, 4, 1);
        FRead(RC->stream, picheight, 4, 1);
        RC->linenumber = 0;
        RC->width = *picwidth;
        return RC;
      }
      Close(RC->stream);
    }
    else DumpError(RC_UNFOUNDFILE, filename);
    FreePebbleHandle(RC);
  }
  else DumpError(RC_NOMEMORY, "PebbleHandle");
  return NULL;
}


/*
** Denne lille finesse har endnu ikke fundet anvendelse, men den er rar at have
** hvis det nu skulle vise sig... Bemrk at CloseLoader() bliver kaldt efter
** AbortLoader(), s ikke noget med at lukke filer to gange.
*/
void ASM AbortLoader(A0 struct PebbleHandle *data) {}


/*
** Ikke de store overraskelser her :o)
*/
void CloseLoader(A0 struct PebbleHandle *data)
{
  Close(data->stream);
  FreePebbleHandle(data);
}


unsigned long ASM IdentifyLoader(A0 char *name)
{
  /*
  ** Navnet p loaderen skal holdes p hjest 30 tegn.
  */
  strcpy(name, "RAW loader");
  return EXTERN_fileloader;
}


/*
** StormC++ linker kan ikke klare sig uden denne her :o(
*/
void exit(int x) {}

/* End Of File.*/
